# -*- coding: utf-8 -*-
from django.conf.urls.defaults import patterns, url, include
from django.http import HttpResponse
from simple_sso.server import SimpleSSOServer

test_server = SimpleSSOServer()

urlpatterns = patterns('',
    url('^server/', include(test_server.get_urls())),
    url('^client/', include('simple_sso.sso_client.urls')),
    url(r'^login/$', 'django.contrib.auth.views.login', {'template_name': 'admin/login.html'}),
    url('^$', lambda request: HttpResponse('home'), name='root')
)


# -*- coding: utf-8 -*-
from django.conf.urls.defaults import patterns, url
from django.contrib import admin
from django.contrib.admin.options import ModelAdmin
from django.core.urlresolvers import reverse
from django.http import (HttpResponse, HttpResponseForbidden, 
    HttpResponseBadRequest, HttpResponseRedirect)
from django.utils import simplejson
from django.views.generic.base import View
from simple_sso.signatures import build_signature
from simple_sso.sso_server.forms import (RequestTokenRequestForm, AuthorizeForm, 
    VerificationForm)
from simple_sso.sso_server.models import Token, Client
from simple_sso.utils import SIMPLE_KEYS
from urlparse import urljoin
import datetime
import urllib


class RequestTokenView(View):
    """
    Request Token Request view called by the client application to obtain a
    one-time Request Token.
    """
    form_class = RequestTokenRequestForm
    server = None
    
    def get(self, request):
        self.form = self.form_class(self.request.GET)
        if self.form.is_valid():
            return self.form_valid()
        else:
            return self.form_invalid()
    
    def get_token(self):
        return Token.objects.create_for_client(self.form.client)
    
    def form_valid(self):
        token = self.get_token()
        params = [('request_token', token.request_token)]
        signature = build_signature(params, token.client.secret)
        params.append(('signature', signature))
        data = urllib.urlencode(params)
        return HttpResponse(data)
    
    def form_invalid(self):
        if self.form.invalid_signature:
            return HttpResponseForbidden()
        return HttpResponseBadRequest()


class AuthorizeView(View):
    """
    The client get's redirected to this view with the `request_token` obtained
    by the Request Token Request by the client application beforehand.
    
    This view checks if the user is logged in on the server application and if
    that user has the necessary rights.
    
    If the user is not logged in, the user is prompted to log in.
    """
    form_class = AuthorizeForm
    server = None
    
    def get(self, request):
        self.form = self.form_class(request.GET)
        if self.form.is_valid():
            self.token = self.form.cleaned_data['token']
            if self.check_token_timeout():
                return self.form_valid()
            else:
                return self.token_timeout()
        else:
            return self.form_invalid()
        
    def form_valid(self):
        if self.request.user.is_authenticated():
            return self.form_valid_authenticated()
        else:
            return self.form_valid_unauthenticated()
        
    def check_token_timeout(self):
        delta = datetime.datetime.now() - self.token.created
        if delta > self.server.token_timeout:
            return False
        else:
            return True
    
    def token_timeout(self):
        self.token.delete()
        return HttpResponseForbidden()
    
    def form_valid_authenticated(self):
        if self.has_access():
            return self.success()
        else:
            return self.access_denied()
        
    def has_access(self):
        return True
    
    def success(self):
        url = urljoin(self.token.client.root_url, 'authenticate') + '/'
        params = [('request_token', self.token.request_token), ('auth_token', self.token.auth_token)]
        signature = build_signature(params, self.token.client.secret)
        params.append(('signature', signature))
        self.token.user = self.request.user
        self.token.save()
        return HttpResponseRedirect('%s?%s' % (url, urllib.urlencode(params)))
    
    def access_denied(self):
        return HttpResponseForbidden()
    
    def form_valid_unauthenticated(self):
        params = urllib.urlencode([('next', '%s?%s' % (self.request.path, urllib.urlencode(self.request.GET)))])
        return HttpResponseRedirect('%s?%s' % (reverse('django.contrib.auth.views.login'), params))
        
    def form_invalid(self):
        if self.form.invalid_signature:
            return HttpResponseForbidden()
        return HttpResponseBadRequest()


class VerifyView(View):
    """
    View called by the client application to verify the Auth Token passed by
    the client request as GET parameter with the server application
    """
    form_class = VerificationForm
    server = None
    
    def get(self, request):
        self.form = self.form_class(request.GET)
        if self.form.is_valid():
            return self.form_valid()
        else:
            return self.form_invalid()
    
    def construct_user(self):
        data = {}
        for key in SIMPLE_KEYS:
            data[key] = getattr(self.token.user, key)
        data['permissions'] = []
        for perm in self.token.user.user_permissions.select_related('content_type').all():
            data['permissions'].append({
                'content_type': perm.content_type.natural_key(),
                'codename': perm.codename,
            })
        return data

    def get_user_json(self):
        """
        Returns the JSON string representation of the user object for a client.
        """
        data = self.construct_user()
        return simplejson.dumps(data)
    
    def form_valid(self):
        self.token = self.form.cleaned_data['token']
        self.user = self.get_user_json()
        params = [('user', self.user)]
        signature = build_signature(params, self.token.client.secret)
        params.append(('signature', signature))
        data = urllib.urlencode(params)
        self.token.delete()
        return HttpResponse(data)
        
    def form_invalid(self):
        if self.form.invalid_signature:
            return HttpResponseForbidden()
        return HttpResponseBadRequest()


class SimpleSSOServer(object):
    request_token_view = RequestTokenView
    authorize_view = AuthorizeView
    verify_view = VerifyView
    token_timeout = datetime.timedelta(minutes=30)
    client_admin = ModelAdmin
    
    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.register_admin()
        
    def register_admin(self):
        admin.site.register(Client, self.client_admin)
    
    def get_urls(self):
        return patterns('simple_sso.sso_server.views',
            url(r'^request-token/$', self.request_token_view.as_view(server=self), name='simple-sso-request-token'),
            url(r'^authorize/$', self.authorize_view.as_view(server=self), name='simple-sso-authorize'),
            url(r'^verify/$', self.verify_view.as_view(server=self), name='simple-sso-verify'),
        )



# -*- coding: utf-8 -*-
import hashlib
import hmac
import urllib


def build_signature(parameters, secret):
    """
    Builds the signature for the parameters and the body given.
    
    Parameters is a list of tuples.
    """
    message = urllib.urlencode(sorted(parameters))
    return hmac.new(secret.encode('ascii'), message.encode('ascii'), hashlib.sha256).hexdigest()

def verify_signature(parameters, signature, secret):
    """
    Verifies the signature
    
    Parameters is a list of tuples.
    """
    result = 0
    built_signature = build_signature(parameters, secret)
    if len(signature) != len(built_signature):
        return False
    for x, y in zip(built_signature, signature):
        result |= ord(x) ^ ord(y)
    return result == 0



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models.query_utils import Q
from simple_sso.utils import gen_secret_key
import datetime


def gen_client_key(field):
    """
    Helper function to give default values to Client.secret and Client.key
    """
    def _genkey():
        key = gen_secret_key(64)
        while Client.objects.filter(**{field: key}).exists():
            key = gen_secret_key(64)
        return key
    return _genkey


class TokenManager(models.Manager):
    def create_for_client(self, client):
        """
        Create a new token for a client object.
        """
        request_token = gen_secret_key(64)
        auth_token = gen_secret_key(64)
        # check unique constraints
        while self.filter(Q(request_token=request_token) | Q(auth_token=auth_token)).exists():
            request_token = gen_secret_key(64)
            auth_token = gen_secret_key(64)
        return self.create(
            client=client,
            request_token=request_token,
            auth_token=auth_token,
        )


class Client(models.Model):
    """
    A client. If secret/key change, the client website has to be updated too!
    """
    secret = models.CharField(max_length=64, unique=True, default=gen_client_key('secret'))
    key = models.CharField(max_length=64, unique=True, default=gen_client_key('key'))
    root_url = models.URLField(verify_exists=False)
    
    def __unicode__(self):
        return self.root_url
    
    def rotate_keys(self):
        self.secret = gen_client_key('secret')()
        self.key = gen_client_key('key')()
        self.save()


class Token(models.Model):
    """
    An auth token used to authenticate a user.
    """
    client = models.ForeignKey(Client)
    request_token = models.CharField(max_length=64, unique=True)
    auth_token = models.CharField(max_length=64, unique=True)
    user = models.ForeignKey('auth.User', null=True)
    created = models.DateTimeField(default=datetime.datetime.now)
    
    objects = TokenManager()



# -*- coding: utf-8 -*-
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.utils import simplejson
from simple_sso.utils import SIMPLE_KEYS

def load_json_user(json):
    """
    Given a JSON string, returns a Django User instance.
    """
    data = simplejson.loads(json)
    try:
        user = User.objects.get(username=data['username'])
    except User.DoesNotExist:
        user = User()
    
    for key in SIMPLE_KEYS:
        setattr(user, key, data[key])
    user.set_unusable_password()
    user.save()
    
    ctype_cache = {}
    
    permissions = []
    
    for perm in data['permissions']:
        ctype = ctype_cache.get(perm['content_type'], None)
        if not ctype:
            try:
                ctype = ContentType.objects.get_by_natural_key(perm['content_type'])
            except ContentType.DoesNotExist:
                continue
            ctype_cache[perm['content_type']] = ctype
        try:
            permission = Permission.objects.get(content_type=ctype, codename=perm['codename'])
        except Permission.DoesNotExist:
            continue
        permissions.append(permission)
    
    user.user_permissions = permissions
    return user



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http import (HttpResponseRedirect, HttpResponse, 
    HttpResponseForbidden, HttpResponseBadRequest)
from django.test.client import Client as TestClient, RequestFactory
from django.test.testcases import TestCase
from django.utils import simplejson
from simple_sso.server import AuthorizeView
from simple_sso.signatures import build_signature
from simple_sso.sso_client.utils import load_json_user
from simple_sso.sso_server.models import Client, Token
from simple_sso.test_urls import test_server
from simple_sso.test_utils.context_managers import (SettingsOverride, 
    UserLoginContext)
from simple_sso.utils import SIMPLE_KEYS, gen_secret_key
import datetime
import urlparse



class SimpleSSOTests(TestCase):
    urls = 'simple_sso.test_urls'

    def setUp(self):
        import requests
        def get(url, params={}, headers={}, cookies=None, auth=None, **kwargs):
            testclient = TestClient()
            return testclient.get(url, params)
        requests.get = get
        self.client.logout()
    
    def test_walkthrough(self):
        # create a user and a client
        USERNAME = PASSWORD = 'myuser'
        server_user = User.objects.create_user(USERNAME, 'my@user.com', PASSWORD)
        client = Client.objects.create(root_url='/client/')
        with SettingsOverride(SIMPLE_SSO_KEY=client.key, SIMPLE_SSO_SECRET=client.secret):
            # verify theres no tokens yet
            self.assertEqual(Token.objects.count(), 0)
            response = self.client.get(reverse('simple-sso-login'))
            # there should be a token now
            self.assertEqual(Token.objects.count(), 1)
            # this should be a HttpResponseRedirect
            self.assertEqual(response.status_code, HttpResponseRedirect.status_code)
            # check that it's the URL we expect
            url = urlparse.urlparse(response['Location']) 
            path = url.path
            self.assertEqual(path, reverse('simple-sso-authorize'))
            # follow that redirect
            response = self.client.get(response['Location'])
            # now we should have another redirect to the login
            self.assertEqual(response.status_code, HttpResponseRedirect.status_code)
            # check that the URL is correct
            url = urlparse.urlparse(response['Location']) 
            path = url.path
            self.assertEqual(path, reverse('django.contrib.auth.views.login'))
            # follow that redirect
            login_url = response['Location']
            response = self.client.get(login_url)
            # now we should have a 200
            self.assertEqual(response.status_code, HttpResponse.status_code)
            # and log in using the username/password from above
            response = self.client.post(login_url, {'username': USERNAME, 'password': PASSWORD})
            # now we should have a redirect back to the authorize view
            self.assertEqual(response.status_code, HttpResponseRedirect.status_code)
            # check that it's the URL we expect
            url = urlparse.urlparse(response['Location']) 
            path = url.path
            self.assertEqual(path, reverse('simple-sso-authorize'))
            # follow that redirect
            response = self.client.get(response['Location'])
            # this should again be a redirect
            self.assertEqual(response.status_code, HttpResponseRedirect.status_code)
            # this time back to the client app, confirm that!
            url = urlparse.urlparse(response['Location']) 
            path = url.path
            self.assertEqual(path, reverse('simple-sso-authenticate'))
            # follow it again
            response = self.client.get(response['Location'])
            # again a redirect! This time to /
            url = urlparse.urlparse(response['Location']) 
            path = url.path
            self.assertEqual(path, reverse('root'))
            # if we follow to root now, we should be logged in
            response = self.client.get(response['Location'])
            client_user = get_user(self.client)
            self.assertEqual(client_user.password, '!')
            self.assertNotEqual(server_user.password, '!')
            for key in SIMPLE_KEYS:
                self.assertEqual(getattr(client_user, key), getattr(server_user, key))
    
    def test_user_already_logged_in(self):
        # create a user and a client
        USERNAME = PASSWORD = 'myuser'
        server_user = User.objects.create_user(USERNAME, 'my@user.com', PASSWORD)
        client = Client.objects.create(root_url='/client/')
        with SettingsOverride(SIMPLE_SSO_KEY=client.key, SIMPLE_SSO_SECRET=client.secret):
            with UserLoginContext(self, server_user):
                # try logging in and auto-follow all 302s
                self.client.get(reverse('simple-sso-login'), follow=True)
                # check the user
                client_user = get_user(self.client)
                self.assertEqual(client_user.password, '!')
                self.assertNotEqual(server_user.password, '!')
                for key in SIMPLE_KEYS:
                    self.assertEqual(getattr(client_user, key), getattr(server_user, key))
    
    def test_custom_keygen(self):
        # WARNING: The following test uses a key generator function that is
        # highly insecure and should never under any circumstances be used in
        # a production enivornment
        with SettingsOverride(SIMPLE_SSO_KEYGENERATOR=lambda length: 'test'):
            self.assertEqual(gen_secret_key(40), 'test')
            
    def test_load_json_user(self):
        userdata = {
            'username': 'mytestuser',
            'password': 'testpassword',
            'first_name': 'mytestuser',
            'last_name': 'mytestuser',
            'email': 'mytestuser@example.com',
            'is_staff': True,
            'is_superuser': False,
            'permissions': []
        }
        jsondata = simplejson.dumps(userdata)
        user = load_json_user(jsondata)
        for key in SIMPLE_KEYS:
            self.assertEqual(getattr(user, key), userdata[key])
        self.assertFalse(user.check_password('testpassword'))
    
    def test_request_token_view_invalid_signature(self):
        client = Client.objects.create(root_url='/client/')
        data = {'key': client.key, 'signature': 'x' * 64}
        response = self.client.get(reverse('simple-sso-request-token'), data)
        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
    
    def test_request_token_view_bad_request(self):
        data = {'key': 'y' * 64, 'signature': 'x' * 64}
        response = self.client.get(reverse('simple-sso-request-token'), data)
        self.assertEqual(response.status_code, HttpResponseBadRequest.status_code)
    
    def test_authorize_view_token_timeout(self):
        client = Client.objects.create(root_url='/client/')
        token = Token.objects.create_for_client(client)
        token.created = datetime.datetime.now() - test_server.token_timeout - datetime.timedelta(hours=1) 
        token.save()
        data = {
            'key': client.key,
            'request_token': token.request_token,
        }
        data['signature'] = build_signature(data.items(), client.secret)
        response = self.client.get(reverse('simple-sso-authorize'), data)
        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
    
    def test_authorize_view_invalid_request_token(self):
        client = Client.objects.create(root_url='/client/')
        data = {
            'key': client.key,
            'request_token': 'x' * 64,
        }
        data['signature'] = build_signature(data.items(), client.secret)
        response = self.client.get(reverse('simple-sso-authorize'), data)
        self.assertEqual(response.status_code, HttpResponseBadRequest.status_code)
    
    def test_authorize_view_invalid_signature(self):
        client = Client.objects.create(root_url='/client/')
        token = Token.objects.create_for_client(client)
        token.created = datetime.datetime.now() - test_server.token_timeout - datetime.timedelta(hours=1) 
        token.save()
        data = {
            'key': client.key,
            'request_token': token.request_token,
            'signature': 'x' * 64
        }
        response = self.client.get(reverse('simple-sso-authorize'), data)
        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)
    
    def test_authorize_view_no_access(self):
        class NoAccessAuthorizeView(AuthorizeView):
            server = test_server
            
            def has_access(self):
                return False
        client = Client.objects.create(root_url='/client/')
        token = Token.objects.create_for_client(client)
        data = {
            'key': client.key,
            'request_token': token.request_token,
        }
        data['signature'] = build_signature(data.items(), client.secret)
        request = RequestFactory().get(reverse('simple-sso-authorize'), data)
        USERNAME = PASSWORD = 'myuser'
        server_user = User.objects.create_user(USERNAME, 'my@user.com', PASSWORD)
        request.user = server_user
        view = NoAccessAuthorizeView.as_view()
        response = view(request)
        self.assertEqual(response.status_code, HttpResponseForbidden.status_code)



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('simple_sso.sso_client.views',
    url(r'^$', 'login_view', name='simple-sso-login'),
    url(r'^authenticate/$', 'authenticate_view', name='simple-sso-authenticate'),
)



# -*- coding: utf-8 -*-
from django import forms
from simple_sso.signatures import verify_signature
from simple_sso.sso_server.models import Token, Client


class BaseForm(forms.Form):
    """
    Base Simple SSO form that verifies the key and signature.
    """
    signature = forms.CharField(max_length=64, min_length=64)
    key = forms.CharField(max_length=64)
    
    def __init__(self, *args, **kwargs):
        self.invalid_signature = False
        self.client = None
        return super(BaseForm, self).__init__(*args, **kwargs)
    
    def clean(self):
        data = super(BaseForm, self).clean()
        parameters = [(key, value) for key, value in data.items() if key != 'signature']
        client_key = data['key']
        try:
            self.client = Client.objects.get(key=client_key)
        except Client.DoesNotExist:
            raise forms.ValidationError('Invalid client key')
        secret = self.client.secret
        signature = data['signature']
        if not verify_signature(parameters, signature, secret):
            self.invalid_signature = True
            raise forms.ValidationError('Invalid signature')
        return data


class RequestTokenRequestForm(BaseForm):
    """
    Form used to request a Request Token
    """


class AuthorizeForm(BaseForm):
    """
    Authorize form used to retrieve a Auth Token using a Request Token
    """
    request_token = forms.CharField(max_length=64, min_length=64)
    
    def clean(self):
        data = super(AuthorizeForm, self).clean()
        request_token = data['request_token']
        try:
            token = Token.objects.get(request_token=request_token, client=self.client, user__isnull=True)
        except Token.DoesNotExist:
            raise forms.ValidationError('Invalid request token')
        data['token'] = token
        return data


class VerificationForm(BaseForm):
    """
    Form used to verify a Auth Token
    """
    auth_token = forms.CharField(max_length=64, min_length=64)
    
    def clean(self):
        data = super(VerificationForm, self).clean()
        auth_token = data['auth_token']
        try:
            token = Token.objects.get(auth_token=auth_token, user__isnull=False, client=self.client)
        except Token.DoesNotExist:
            raise forms.ValidationError('Invalid auth token')
        data['token'] = token
        return data



from setuptools import setup, find_packages
from simple_sso import __version__ as version
import os

README = os.path.join(os.path.dirname(__file__), 'README.rst')

with open(README) as fobj:
    long_description = fobj.read()

setup(name="django-simple-sso",
    version=version,
    description="Simple SSO for Django",
    long_description=long_description,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='django sso',
    author='Jonas Obrist',
    author_email='jonas.obrist@divio.ch',
    url='http://github.com/ojii/django-simple-sso',
    license='BSD',
    packages=find_packages(),
    install_requires=['Django>=1.3', 'django-load', 'requests', 'south'],
    include_package_data=True,
    zip_safe=False
)



# -*- coding: utf-8 -*-
from django.conf import settings

class NULL:
    pass

class SettingsOverride(object):
    """
    Overrides Django settings within a context and resets them to their inital
    values on exit.
    
    Example:
    
        with SettingsOverride(DEBUG=True):
            # do something
    """
    
    def __init__(self, **overrides):
        self.overrides = overrides
        
    def __enter__(self):
        self.old = {}
        for key, value in self.overrides.items():
            self.old[key] = getattr(settings, key, NULL)
            setattr(settings, key, value)
        
    def __exit__(self, type, value, traceback):
        for key, value in self.old.items():
            if value is not NULL:
                setattr(settings, key, value)
            else:
                delattr(settings, key) # do not pollute the context!


class UserLoginContext(object):
    def __init__(self, testcase, user):
        self.testcase = testcase
        self.user = user
        
    def __enter__(self):
        loginok = self.testcase.client.login(username=self.user.username, 
                                             password=self.user.username)
        self.old_user = getattr(self.testcase, 'user', None)
        self.testcase.user = self.user
        self.testcase.assertTrue(loginok)
        
    def __exit__(self, exc, value, tb):
        self.testcase.user = self.old_user
        if not self.testcase.user:
            delattr(self.testcase, 'user')
        self.testcase.client.logout()



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.auth import login
from django.contrib.auth.backends import ModelBackend
from django.http import HttpResponseBadRequest, QueryDict, HttpResponseRedirect
from simple_sso.signatures import build_signature, verify_signature
from simple_sso.sso_client.utils import load_json_user
from urlparse import urljoin, urlparse
import requests
import urllib

BACKEND = ModelBackend()

def get_request_token():
    """
    Requests a Request Token from the SSO Server. Returns False if the request
    failed.
    """
    params = [('key', settings.SIMPLE_SSO_KEY)]
    signature = build_signature(params, settings.SIMPLE_SSO_SECRET)
    params.append(('signature', signature))
    url = urljoin(settings.SIMPLE_SSO_SERVER, 'request-token') + '/'
    response = requests.get(url, params=dict(params))
    if response.status_code != 200:
        return False
    data = QueryDict(response.content)
    if 'signature' not in data:
        return False
    if 'request_token' not in data:
        return False
    params = [(key, value) for key,value in data.items() if key != 'signature']
    if not verify_signature(params, data['signature'], settings.SIMPLE_SSO_SECRET):
        return False
    return data['request_token']

def verify_auth_token(data):
    """
    Verifies a Auth Token in a QueryDict. Returns a
    django.contrib.auth.models.User instance if successful or False.
    """
    if 'auth_token' not in data:
        return False
    if 'request_token' not in data:
        return False
    auth_token = data['auth_token']
    params = [('auth_token', auth_token), ('key', settings.SIMPLE_SSO_KEY)]
    signature = build_signature(params, settings.SIMPLE_SSO_SECRET)
    params.append(('signature', signature))
    url = urljoin(settings.SIMPLE_SSO_SERVER, 'verify') + '/'
    response = requests.get(url, params=dict(params))
    if response.status_code != 200:
        return False
    data = QueryDict(response.content)
    if 'signature' not in data:
        return False
    if 'user' not in data:
        return False
    params = [(key, value) for key,value in data.items() if key != 'signature']
    if not verify_signature(params, data['signature'], settings.SIMPLE_SSO_SECRET):
        return False
    return load_json_user(data['user'])

def get_next(request):
    """
    Given a request, returns the URL where a user should be redirected to 
    after login. Defaults to '/'
    """
    next = request.GET.get('next', None)
    if not next:
        return '/'
    netloc = urlparse(next)[1]

    # Heavier security check -- don't allow redirection to a different
    # host.
    # Taken from django.contrib.auth.views.login
    if netloc and netloc != request.get_host():
        return '/'
    return next

def login_view(request):
    """
    Login view.
    
    Requests a Request Token and then redirects the User to the the SSO Server.
    """
    next = get_next(request)
    request.session['simple-sso-next'] = next
    request_token = get_request_token()
    if not request_token:
        return HttpResponseBadRequest()
    params = [('request_token', request_token), ('key', settings.SIMPLE_SSO_KEY)]
    signature = build_signature(params, settings.SIMPLE_SSO_SECRET)
    params.append(('signature', signature))
    query_string = urllib.urlencode(params)
    url = urljoin(settings.SIMPLE_SSO_SERVER, 'authorize') + '/'
    return HttpResponseRedirect('%s?%s' % (url, query_string))
    
def authenticate_view(request):
    """
    Authentication view.
    
    Verifies the user token and logs the user in.
    """
    user = verify_auth_token(request.GET)
    if not user:
        return HttpResponseBadRequest()
    user.backend = "%s.%s" % (BACKEND.__module__, BACKEND.__class__.__name__)
    login(request, user)
    return HttpResponseRedirect(request.session.get('simple-sso-next', '/'))
